function stringdemo

data.nDisp = 5; % number of harmonics to display

% plot gui figure
MainFigure = figure('Menubar', 'none', 'numbertitle',...
    'off', 'name', 'String Demo');

% string shape
a = axes('parent', MainFigure, 'units', 'pixels', 'tag', 'axWaveform',...
    'NextPlot', 'ReplaceChildren', 'yTick', 0, 'yGrid', 'on');
xlabel(a, 'x [m]');
ylabel(a, 'u(x,t) [m]');
hold(a, 'on');
plot(a, 0, NaN, 'Color', 'black', 'LineWidth', 2, 'Tag', 'linWaveform');
plot(a, 0, NaN, 'r.', 'MarkerSize', 20, 'Tag', 'mrkWaveform');
% harmonics
for i = 1 : data.nDisp
    a = axes('parent', MainFigure, 'units', 'pixels', 'tag', 'axHarmonics',...
        'NextPlot', 'ReplaceChildren', 'xTick', [], 'yTick', 0, 'yGrid', 'on');
    ylabel(a, 'u(x,t) [m]');
    hold(a, 'on');
    plot(a, 0, NaN, 'Color', 'blue', 'LineWidth', 2, 'Tag', 'linHarmonics');
    plot(a, 0, NaN, 'r.', 'MarkerSize', 20, 'Tag', 'mrkHarmonics');
end
% string time history
a = axes('parent', MainFigure, 'units', 'pixels', 'tag', 'axDispTime',...
    'NextPlot', 'ReplaceChildren', 'yTick', 0, 'yGrid', 'on');
xlabel(a, 't [s]');
ylabel(a, 'u(x0,t) [m]');
hold(a, 'on');
plot(a, 0, NaN, 'Color', 'black', 'LineWidth', 2, 'Tag', 'linDispTime');
plot(a, 0, NaN, 'r.', 'MarkerSize', 20, 'Tag', 'mrkDispTime');
% force time history
a = axes('parent', MainFigure, 'units', 'pixels', 'tag', 'axForceTime',...
    'NextPlot', 'ReplaceChildren', 'yTick', 0, 'yGrid', 'on');
ylabel(a, 'f(0,t) [N]');
hold(a, 'on');
plot(a, 0, NaN, 'Color', 'black', 'LineWidth', 2, 'Tag', 'linForceTime');
plot(a, 0, NaN, 'r.', 'MarkerSize', 20, 'Tag', 'mrkForceTime');
%
% force time history
axes('parent', MainFigure, 'units', 'pixels', 'tag', 'axSpect',...
    'NextPlot', 'ReplaceChildren', 'yTick', 0:20:80, 'yTickLabel', cellstr(num2str((-80:20:0)')));

% reset button
uicontrol('parent', MainFigure, 'Style', 'pushbutton', 'tag', 'butReset', ...
    'String', 'Reset', 'Callback', @(src,event)butCallback(src, 'reset'),...
    'units', 'pixels', 'position', [0 0 100 20]);

% start/stop button
uicontrol('parent', MainFigure, 'Style', 'pushbutton', 'tag', 'butStart', ...
    'String', 'Start', 'Callback', @(src,event)butCallback(src, 'startstop'),...
    'units', 'pixels', 'position', [0 0 100 20]);

% start/stop button
uicontrol('parent', MainFigure, 'Style', 'pushbutton', 'tag', 'butPlayback', ...
    'String', 'Playback', 'Callback', @(src,event)butPlaybackCallback(src),...
    'units', 'pixels', 'position', [0 0 100 20]);

t = uitable('parent', MainFigure, 'Tag', 'tabString', ...
    'RowName', [], ...
    'ColumnName', {'mass [kg/m]', 'Tension [N]', 'damping [Ns/m2]', 'length [L]'}, ...
    'ColumnFormat', {'numeric', 'numeric', 'numeric', 'numeric'}, ...
    'ColumnEditable', true(1,4), ...
    'Data', {1e-3, 70, 1e-2, 1}, ...
    'CellEditCallback', @(src,event)compprop(src));
set(t, 'position', get(t, 'extent'));

t = uitable('parent', MainFigure, 'Tag', 'tabExc', ...
    'RowName', [], ...
    'ColumnName', {'type', 'exc. pos. [m]', 'resp. pos. [m]'}, ...
    'ColumnFormat', {{'struck', 'pluck'}, 'numeric', 'numeric'}, ...
    'ColumnEditable', true(1,3), ...
    'Data', {'pluck', .1, .2}, ...
    'CellEditCallback', @(src,event)Reset(src));
set(t, 'position', get(t, 'extent'));

t = uitable('parent', MainFigure, 'Tag', 'tabMod', ...
    'RowName', [], ...
    'ColumnName', {'#modes', '#period'}, ...
    'ColumnFormat', {'numeric', 'numeric'}, ...
    'ColumnEditable', true(1,2), ...
    'Data', {100, 5}, ...
    'CellEditCallback', @(src,event)Reset(src));
set(t, 'position', get(t, 'extent'));

t = uitable('parent', MainFigure, 'Tag', 'tabProp', ...
    'RowName', [], ...
    'ColumnName', {'fund. freq [Hz]', 'speed [m/s]', 'time cnst [s]'}, ...
    'ColumnFormat', {'numeric', 'numeric', 'numeric'}, ...
    'ColumnEditable', true(1,3), ...
    'Data', {0, 0, 0});
set(t, 'position', get(t, 'extent'));

% gui data variables
data.Nx = 101;
data.Nt = 201;

guidata(MainFigure, data); % store data into gui

ResizeFunction(MainFigure);
Reset(MainFigure);

set(MainFigure, 'ResizeFcn', @(src,event)ResizeFunction(src));
end

function compprop(src)
Reset(src);
h = guihandles(src);
d = get(h.tabString, 'data');
mu = d{1};
T = d{2};
beta = d{3};
L = d{4};

c = sqrt(T/mu);
f1 = c/(2*L);
tau = mu/(2*beta);
set(h.tabProp, 'Data', {f1, c, tau});
end

function Stop(src)
data = guidata(src);
h = guihandles(src);
data.isRunning = false;
set(h.butStart, 'String', 'Start');
guidata(src, data);
end

function Reset(src)
data = guidata(src);
h = guihandles(src);

d = get(h.tabString, 'Data');
data.string.mu = d{1};
data.string.T = d{2};
data.string.beta = d{3};
data.string.L = d{4};

data.exc.F0 = 1;
d = get(h.tabExc, 'Data');
data.exc.type = d{1};
data.exc.x0 = d{2};
r = d{3};

d = get(h.tabMod, 'Data');
data.nMod = d{1};
data.nPer = d{2};

[data.t, data.x, data.ut, data.ux, Un] = TimeHistory(data.string, data.exc, data.nMod, data.Nx, data.Nt, data.nPer);
data.ft_0 = data.string.T * ((data.ux(:,2)-data.ux(:,1)) .' * data.ut) / (data.x(2)-data.x(1));
[trash, data.iX] = min(abs(data.x-r));

data.ut_x0 = data.ux(:,data.iX).' * data.ut;
data.ux_t0 = data.ut(:,1).' * data.ux;

Max = max(max(abs((data.ut.' * data.ux))));

Un = 20*log10(abs(Un./Un(1)))+80;
Un(Un < 0) = NaN;
stem(h.axSpect, 1:length(Un), Un);
set(h.axDispTime, 'ylim', [-1 1]*Max, 'xlim', [min(data.t) max(data.t)], ...
    'xTick', data.t(1:data.Nt:end), 'xGrid', 'on');
set(h.axForceTime, 'ylim', [-1 1].*max(abs(data.ft_0)), 'xlim', [min(data.t) max(data.t)], ...
    'xTick', data.t(1:data.Nt:end), 'xTickLabel', [], 'xGrid', 'on');
set(h.axWaveform, 'ylim', [-1 1]*Max, 'xlim', [min(data.x) max(data.x)], ...
    'xTick', unique(sort([0 data.exc.x0 r max(data.x)])));

Max = 0;
for i = 1 : data.nDisp
    Max = max(Max, max(max(abs((data.ut(i,:).' *data. ux(i,:))))));
end
set(h.axHarmonics, 'ylim', [-1 +1]*Max, 'xLim', [min(data.x), max(data.x)]);

data.player = audioplayer(1, 44100);

data.q = 1;
data.isRunning = false;
set(h.butStart, 'String', 'Start');

guidata(src, data);

plotframe(src);
end


function plotframe(src)

data = guidata(src);
h = guihandles(src);
q = data.q;
iX = data.iX;
t = data.t;
x = data.x;
ut = data.ut;
ux = data.ux;
ut_x0 = data.ut_x0;
ft_0 = data.ft_0;

set(h.linDispTime, 'xData', t(1:q), 'yData', ut_x0(1:q));
set(h.mrkDispTime, 'xData', t(q), 'yData', ut_x0(q));

set(h.linForceTime, 'xData', t(1:q), 'yData', ft_0(1:q));
set(h.mrkForceTime, 'xData', t(q), 'yData', ft_0(q));

set(h.linWaveform, 'xData', x, 'yData', ut(:,q).' * ux);
set(h.mrkWaveform, 'xData', x(iX), 'yData', ut(:,q).' * ux(:,iX));

for i = 1 : data.nDisp
    set(h.linHarmonics(i), 'xData', x, 'yData', ut(i,q).' * ux(i,:));
    set(h.mrkHarmonics(i), 'xData', x(iX), 'yData', ut(i,q).' * ux(i,iX));
end
end

function Play(src)
data = guidata(src);
h = guihandles(src);
set(h.butStart, 'String', 'Stop');
data.isRunning = true;
guidata(src, data);

while data.q <= length(data.t) && data.isRunning
    plotframe(src);
    data.q = data.q + 1;
    guidata(src, data);
    
    drawnow;
    
    data = guidata(src);
end

end

function butPlaybackCallback(src)

data = guidata(src);
stop(data.player);
if (get(data.player, 'TotalSamples') == 1)
    fs = get(data.player, 'SampleRate');
    u = PlaybackHistory(data.string, data.exc, data.nMod, 4, fs);
    u = .9*u/max(abs(u));
    data.player = audioplayer(u, fs);
end
play(data.player);
guidata(src, data);

end

function butCallback(src, mode)
data = guidata(src);
if data.isRunning
    switch mode
        case 'reset'
            Reset(src);
        case 'startstop'
            Stop(src);
    end
else
    switch mode
        case 'reset'
            Reset(src);
        case 'startstop'
            Play(src);
    end
end
end

function ResizeFunction(src)
Frame = 20;
Sep = 10;
h = guihandles(src);

AlignItems(h.axWaveform, 'left', 2*Frame);
AlignItems(h.axWaveform, 'bottom', 2*Frame);
AlignItems([h.axWaveform h.axHarmonics], 'left');
AlignItems([h.axWaveform h.axHarmonics], 'stretch up', Frame, Sep);

AlignItems([h.axWaveform h.axDispTime], 'bottom');
AlignItems([h.axWaveform h.axDispTime], 'stretch right', Frame, 3*Frame, [1 2]);

p = get(h.axWaveform, 'position');
W = p(3);
H = p(4);
for i = 1 : length(h.axHarmonics)
    p = get(h.axHarmonics(i), 'position');
    p(3) = W;
    set(h.axHarmonics(i), 'position', p);
end
p = get(h.axDispTime, 'position');
p(4) = H;
set(h.axDispTime, 'position', p);
p = get(h.axForceTime, 'position');
p(4) = H;
set(h.axForceTime, 'position', p);
p = get(h.axSpect, 'position');
p(4) = 2*H;
set(h.axSpect, 'position', p);

AlignItems([h.axDispTime h.axForceTime h.axSpect], 'left');
AlignItems([h.axDispTime h.axForceTime h.axSpect], 'row up', Sep);
AlignItems(h.axForceTime, 'stretch right', Frame);
AlignItems(h.axSpect, 'stretch right', Frame);

AlignItems([h.axSpect h.butReset h.tabExc h.tabString h.tabMod h.tabProp], 'left');
AlignItems([h.axSpect h.butReset h.tabProp], 'row up', Frame);
AlignItems([h.tabProp h.tabMod h.tabExc h.tabString], 'row up', Sep);

AlignItems([h.butReset h.butStart h.butPlayback], 'bottom');
AlignItems([h.butReset h.butStart h.butPlayback], 'row right', Sep);
end


function [t, x, ut, ux, Un] = TimeHistory(string, exc, nMod, Nx, Nt, nPer)

c = sqrt(string.T/string.mu);
tau = string.mu/2/string.beta;
m = string.L/exc.x0;
n = 1:nMod;
kn = n*pi/string.L;

omn = kn*c;

xin = 1./(omn*tau);  % damping
omd = omn.*sqrt(1-xin.^2);

x = (0:Nx-1)/Nx*string.L;
t = (0:nPer*Nt-1)/Nt/(omn(1)/2/pi);

ux = sin(kn.'* x);

switch exc.type
    case 'struck'
        Un = (sin(n*pi/m)./omd);
        ut = exc.F0 * ((Un.'*exp(-t/tau)) .* sin(omd.'* t));
    case 'pluck'
        Un = (2*m^2/pi^2/(m-1)) * (sin(n*pi/m)./n.^2);
        ut = ((Un.'*exp(-t/tau)) .* ...
            (cos(omd.'*t) + diag(xin./(sqrt(1-xin.^2))) * sin(omd.'* t)));
end
end

function [ux] = PlaybackHistory(string, exc, nMod, T, fs)

% time scale for playback
t = 0:1/fs:T;
tau = string.mu/2/string.beta;
t = t(t < 4*log(10)*tau);

c = sqrt(string.T/string.mu);
m = string.L/exc.x0;
n = 1:nMod;
kn = n*pi/string.L;

omn = kn*c;

ind = omn/2/pi < fs/2;

xin = 1./(omn*tau);  % damping
omd = omn.*sqrt(1-xin.^2);

switch exc.type
    case 'struck'
        ut = exc.F0 * (((sin(n*pi/m)./omd).'*exp(-t/tau)) .* sin(omd.'* t));
    case 'pluck'
        ut = (2*m^2/pi^2/(m-1)) * (((sin(n*pi/m)./n.^2).'*exp(-t/tau)) .* ...
            (cos(omd.'*t) + diag(xin./(sqrt(1-xin.^2))) * sin(omd.'* t)));
end

ux = sin(kn(ind)* exc.x0) * ut(ind,:);

end